( cd usr/lib ; rm -rf libquicktime1394.so.0 )
( cd usr/lib ; ln -sf libquicktime1394.so.0.0.0 libquicktime1394.so.0 )
( cd usr/lib ; rm -rf libquicktime1394.so )
( cd usr/lib ; ln -sf libquicktime1394.so.0.0.0 libquicktime1394.so )
( cd usr/lib ; rm -rf libquicktime.so.0 )
( cd usr/lib ; ln -sf libquicktime.so.0.0.0 libquicktime.so.0 )
( cd usr/lib ; rm -rf libquicktime.so )
( cd usr/lib ; ln -sf libquicktime.so.0.0.0 libquicktime.so )
