#ifndef __LQT_VERSION_H_
#define __LQT_VERSION_H_

/*
 *  Codec API version
 *  This should be increased (by one or so) after the codec API
 *  changes. This will happen sometimes during the development
 *  cycle but is hopefully constant afterwards
 */

#define LQT_CODEC_API_VERSION 3

/*
 * libquicktime library version
 */

#define LQT_VERSION "0.9.7"

#endif /* __LQT_VERSION_H_ */
